import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

public class DotPlotPanel extends JPanel 
  implements MouseMotionListener, 
	     ActionListener, 
	     ChangeListener {
  private Rule columnRule;
  private Rule rowRule;
  private ScrollableDotPlot picture;

  DotPlotEngine mDPE;

  String MaxName="This is a test";
  int MaxNameSize;
  JLabel mLabel;
  String mLabelText;
  int mouseX;
  int mouseY;

  int I_MIN = 1;
  int I_MAX = 255;
  int I_INIT = 128;
  double iscale;

  protected static ImageIcon createImageIcon(String path) {
    java.net.URL imgURL = DotPlotPanel.class.getResource(path);
    //error handling omitted for clarity...
    return new ImageIcon(imgURL);
  }

  public DotPlotPanel(DotPlotEngine DPE) {

    mDPE = DPE;
    
    //    int rows = mDPE.GetTotalRows();
    //int cols = mDPE.GetCols();

    int rows = mDPE.GetTotalRows2();
    int cols = mDPE.GetCols2();
    System.out.println("rows="+rows+" cols="+cols);
    int numQueries = mDPE.GetNumQueries();

    int queryseparatorSize=4;
    int pixelsX = rows;
    int pixelsY = cols;

    System.out.println("pixelsX="+pixelsX+" pixelsY="+pixelsY);

    // Create the row and column headers.
    columnRule = new Rule(Rule.HORIZONTAL,mDPE.GetScale());
    columnRule.setPreferredWidth(pixelsY);
    rowRule = new Rule(Rule.VERTICAL,mDPE.GetScale());
    rowRule.setPreferredHeight(pixelsX);
    
    // Set up the scroll pane.
    picture = new ScrollableDotPlot(DPE);
    picture.addMouseMotionListener(this);
    picture.setPreferredSize(new Dimension(pixelsX,pixelsY)); // KJD				 
    JScrollPane pictureScrollPane = new JScrollPane(picture);
    pictureScrollPane.setPreferredSize(new Dimension(300, 300));
    pictureScrollPane.setColumnHeaderView(columnRule);
    pictureScrollPane.setRowHeaderView(rowRule);

    // Add a slider....
    JSlider slider = new JSlider(I_MIN,I_MAX,I_INIT);
    slider.setPreferredSize(new Dimension(80,30));
    slider.addChangeListener(this);
    
    // Create a toolBar and some buttons. 
    JToolBar toolBar = new JToolBar("Still draggable");

    ImageIcon plusIcon = createImageIcon("plus.gif");
    ImageIcon minusIcon = createImageIcon("minus.gif");

    JButton inbutton = new JButton(plusIcon);
    inbutton.addActionListener(this);
    inbutton.setActionCommand("in");

    JButton outbutton = new JButton(minusIcon);
    outbutton.addActionListener(this);
    outbutton.setActionCommand("out");

    toolBar.setBackground(Color.white);
    toolBar.add(inbutton);
    toolBar.add(outbutton);
    toolBar.add(slider,BorderLayout.EAST);

    // Cheap way to limit expansion of JSlider... 
    JPanel space = new JPanel();
    space.setPreferredSize(new Dimension(100,30));
    space.setBackground(Color.white);
    toolBar.add(space);

    toolBar.setPreferredSize(new Dimension(200, 30));

    // Create a label... 
    mLabel = new JLabel();
    mLabel.setForeground(Color.blue);
    mLabel.setText("MegaDot!  Copyright 2004 Baylor College of Medicine");
    
    // A panel to hodl the scroll pane and the toolbar so 
    // that the toolbar can move around the pane without 
    // clobbering the label. 
    JPanel mainPanel = new JPanel();

    mainPanel.setLayout(new BorderLayout());
    mainPanel.add(pictureScrollPane,BorderLayout.CENTER);

    mainPanel.add(mLabel,BorderLayout.SOUTH);

    // Put components in this panel. There is a main panel and a 
    // toolbar, and nothing else, so toolbar can move around the panel. 
    setLayout(new BorderLayout());
    add(toolBar, BorderLayout.PAGE_START);
    add(mainPanel, BorderLayout.CENTER);
    
  }

  public void mouseMoved(MouseEvent e){
    mouseX = e.getX();
    mouseY = e.getY();    
    int scale = mDPE.GetScale();
    int baseX = mouseX * scale;
    int baseY = mouseY * scale; 
    //    mLabelText = "x="+mouseX+" y="+mouseY+"\tbaseX="+baseX+" baseY="+baseY;
    mLabelText = "1 pixel = "+scale+" bp\t\t";
    int fullintensity = (int)(255.0/iscale);
    mLabelText += "Full intensity >= "+fullintensity+" hits.\t\t\t";
    mLabelText += "baseX="+baseX+" baseY="+baseY;
    mLabelText += "\tX="+mouseX+" Y="+mouseY;
    mLabel.setText(mLabelText);      
  }
  
  public void mouseDragged(MouseEvent e){    
  }

  public void actionPerformed(ActionEvent e){
    if ("in".equals(e.getActionCommand())) {
      int scale = mDPE.GetScale();
      scale = scale/2;
      if (scale < 1) scale = 1;
      columnRule.setBasesPerDot(scale);
      rowRule.setBasesPerDot(scale);
      mDPE.SetScale(scale);
      mDPE.ComputeStats();
      mDPE.InitPixels();
      repaint();
    }else{
      int scale = mDPE.GetScale();
      scale = scale*2;
      columnRule.setBasesPerDot(scale);
      rowRule.setBasesPerDot(scale);
      mDPE.SetScale(scale);
      mDPE.ComputeStats();
      mDPE.InitPixels();
      repaint();
    }
  }

  public void stateChanged(ChangeEvent e){
    JSlider source = (JSlider) e.getSource();
    //    if (!source.getValueIsAdjusting()){
      int svalue = (int) source.getValue();
      iscale = ((double)svalue);
      picture.setIScale(iscale);

      int scale = mDPE.GetScale();
      int baseX = mouseX * scale;
      int baseY = mouseY * scale; 
      //    mLabelText = "x="+mouseX+" y="+mouseY+"\tbaseX="+baseX+" baseY="+baseY;
      mLabelText = "1 pixel = "+scale+" bp\t\t";
      int fullintensity = (int)(255.0/iscale);
      mLabelText += "Full intensity >= "+fullintensity+" hits.\t\t\t";
      mLabelText += "baseX="+baseX+" baseY="+baseY;
      mLabel.setText(mLabelText); 
      
      // }
  }
}
